package com.loafle.overflow.central.module.member.service;

import com.loafle.overflow.central.commons.stereotype.WebappAPI;
import com.loafle.overflow.central.commons.utils.EmailSender;
import com.loafle.overflow.central.commons.utils.SessionMetadata;
import com.loafle.overflow.central.module.email.service.EmailAuthService;
import com.loafle.overflow.central.module.member.dao.MemberDAO;

import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.model.apikey.ApiKey;
import com.loafle.overflow.model.domain.Domain;
import com.loafle.overflow.model.domain.DomainMember;
import com.loafle.overflow.model.member.Member;
import com.loafle.overflow.model.probe.Probe;
import com.loafle.overflow.model.meta.MetaMemberStatus;

import com.loafle.overflow.service.central.apikey.ApiKeyService;
import com.loafle.overflow.service.central.domain.DomainMemberService;
import com.loafle.overflow.service.central.member.MemberService;
import com.loafle.overflow.service.central.probe.ProbeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;

import java.net.URLDecoder;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Created by geek on 17. 6. 28.
 */
@Service("MemberService")
public class CentralMemberService implements MemberService {

    @Autowired
    private MemberDAO memberDAO;

    @Autowired
    private EmailAuthService emailAuthService;

    @Autowired
    private EmailSender emailSender;

    @Autowired
    private ApiKeyService apiKeyService;

    @Autowired
    private DomainMemberService domainMemberService;

    @Autowired
    private ProbeService probeService;

    private BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();

    public DomainMember signin(String signinId, String signinPw) throws OverflowException {
        Member m = this.memberDAO.findByEmail(signinId);

        if ( null == m ) {
            throw new OverflowException("SignInIdNotExistException()", new Throwable());
        }

        if ( m.getStatus().getId() == 1 ) {
            throw new OverflowException("EmailNotConfirmedException()", new Throwable());
        }

        Boolean match = passwordEncoder.matches(signinPw, m.getPw());
        if(!match) {
            if (m.getSigninFailCount() > 10) {
                throw new OverflowException("SigninOverFailedException()", new Throwable());
            }
            m.setSigninFailCount(m.getSigninFailCount()+1);
            this.modify(m);
            throw new OverflowException("SignInPwNotMatchException()", new Throwable());
        }

        m.setSigninFailCount(0);
        this.modify(m);

        DomainMember dm = domainMemberService.readByMemberEmail(m.getEmail());

        // Todo Signin History
        return dm;
    }

    public Member signup(Member member, String pw) throws OverflowException {

        Member isMember = this.memberDAO.findByEmail(member.getEmail());

        if (null != isMember && isMember.getId() > 0) {
            throw new OverflowException("JoinedEmailException()", new Throwable());
        }

        boolean checkPass = this.isPasswordStrong(pw);

        if (!checkPass) {
            throw new OverflowException("PasswordNotStrongException()", new Throwable());
//            (
//                    "Passwords must contain at least one uppercase letter, " +
//                            "special character, lowercase letter, and number, " +
//                            "and must be at least 6 characters long.");
        }
        member.setPw(passwordEncoder.encode(pw));

        if (member.getStatus() == null) {
            member.setStatus(new MetaMemberStatus((short) 1));
        }

        Member resMember = this.memberDAO.save(member);

        try {
            this.emailAuthService.sendEmailByMember(member.getId(), member.getEmail());
        } catch (Exception e) {
            // Todo ReSend Mail
            e.printStackTrace();
        }

        return resMember;
    }

    public Member sendEmailForPassword(String email) throws OverflowException {
        Member member = this.memberDAO.findByEmail(email);

        if (null == member) {
            throw new OverflowException("SignInIdNotExistException()", new Throwable());
        }

        try {
            this.emailAuthService.sendEmailResetPassword(member);
        } catch (Exception e) {
            // Todo ReSend Mail
            e.printStackTrace();
        }

        return member;
    }

    public Member resetPassword(String encodeEmail, String pw) throws OverflowException {
        String deStr = null;

        try {
            deStr = URLDecoder.decode(encodeEmail, "UTF-8");
        }catch (Exception e) {

        }

        String deEmail = this.emailSender.decrypt(deStr);
        Member member = this.memberDAO.findByEmail(deEmail);

        if (null == member) {
            throw new OverflowException("", null);
        }

        boolean checkPass = this.isPasswordStrong(pw);

        if (!checkPass) {
            throw new OverflowException("PasswordNotStrongException()", new Throwable());
//                    "Passwords must contain at least one uppercase letter, " +
//                            "special character, lowercase letter, and number, " +
//                            "and must be at least 6 characters long.");
        }
        member.setPw(passwordEncoder.encode(pw));

        return this.modify(member);
    }

    public void signout(Member member) {
        // Todo websocket session remove
    }

    @WebappAPI
    public Member modify(Member member, String pw) throws OverflowException {
        String email = SessionMetadata.getTargetID();
        Member preMember = this.memberDAO.findByEmail(member.getEmail());

        if (null != pw && !pw.equals("")) {
            boolean checkPass = this.isPasswordStrong(pw);

            if (!checkPass) {
                throw new OverflowException("PasswordNotStrongException()", new Throwable());
//                        "Passwords must contain at least one uppercase letter, " +
//                                "special character, lowercase letter, and number, " +
//                                "and must be at least 6 characters long.");
            }

            Boolean match = passwordEncoder.matches(member.getPw(), preMember.getPw());
            if(!match) {
                member.setPw(passwordEncoder.encode(pw));
            }
        } else {
            member.setPw(preMember.getPw());
        }

        if (member.getStatus() == null || member.getStatus().getId() <= 0) {
            member.getStatus().setId(preMember.getStatus().getId());
        }
        return this.modify(member);
    }

    private Member modify(Member member) throws OverflowException {
        return this.memberDAO.save(member);
    }

    public Member confirmPw(String signinId, String signinPw) throws OverflowException {
        Member preMember = this.memberDAO.findByEmail(signinId);

        String encodePw = passwordEncoder.encode(signinPw);
        Boolean match = passwordEncoder.matches(encodePw, preMember.getPw());

        if (!match) {
            throw new OverflowException("SignInPwNotMatchException", new Throwable());
        }

        return preMember;
    }

    public Member forgotPassword(String signinId, String newPw) throws OverflowException {
        Member preMember = this.memberDAO.findByEmail(signinId);

        if (null == preMember) {
            throw new OverflowException("SignInIdNotExistException()", new Throwable());
        }

        Member cMember = this.modify(preMember, newPw);

        return cMember;
    }

    public Member read(long memberId) throws OverflowException {
        if (memberId <= 0) {
            throw new OverflowException("SignInIdNotExistException()", new Throwable());
        }

        Member resMember = this.memberDAO.findOne(memberId);
        return resMember;
    }

    @WebappAPI
    public void withdrawal(Member member) throws OverflowException {
        String email = SessionMetadata.getTargetID();

        // Todo DB delete?
    }

    public List<Member> readAllByProbeKey(String probeKey) throws OverflowException {

        Probe probe = this.probeService.readByProbeKey(probeKey);

        if(probe == null) {
            return null;
        }

        return this.domainMemberService.readAllMemberByDomain(probe.getDomain());
    }

    public List<Member> readAllByApiKey(String apikey) throws OverflowException {

        ApiKey apiKey = this.apiKeyService.readByApiKey(apikey);

        if(apiKey == null) {
            return null;
        }

        return this.domainMemberService.readAllMemberByDomain(apiKey.getDomain());
    }

    public List<Member> readAllByDomain(Domain domain) throws OverflowException {

        return this.domainMemberService.readAllMemberByDomain(domain);
    }

    public List<Member> readAllByDomainID(final long domainID) throws OverflowException {

        return this.domainMemberService.readAllMemberByDomainID(domainID);
    }

    private static final String PASSWORD_REGEXP =  "(" +
            "(?=.*[a-z])" +
            "(?=.*\\d)" +
            "(?=.*[A-Z])" +
            "(?=.*[!@#$%^&*()_+\\-=\\[\\]{};':\"\\\\|,.<>\\/?])" +
            "." +
            "{6,40}" +
            ")";
    private Pattern pattern = Pattern.compile(PASSWORD_REGEXP);

    protected boolean isPasswordStrong(String pass) {
        Matcher m = pattern.matcher(pass);
        return m.matches();
    }
}
