package com.loafle.overflow.central.module.notification.service;

import com.loafle.overflow.central.commons.utils.PageUtil;
import com.loafle.overflow.central.module.notification.dao.NotificationDAO;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.core.model.PageParams;
import com.loafle.overflow.model.member.Member;
import com.loafle.overflow.model.notification.Notification;
import com.loafle.overflow.service.central.notification.NotificationService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * Created by insanity on 17. 8. 25.
 */
@Service("NotificationService")
public class CentralNotificationService implements NotificationService {

    @Autowired
    private NotificationDAO notificationDAO;

    public Notification regist(Notification notification) throws OverflowException {
        return this.notificationDAO.save(notification);
    }

    public Page<Notification> readAllByMember(Member member, PageParams pageParams) throws OverflowException {
        return this.notificationDAO.findAllByMember(member, PageUtil.getPageRequest(pageParams));
    }

    public Page<Notification> readAllUnconfirmedByMember(Member member, PageParams pageParams) throws OverflowException {
        return this.notificationDAO.findAllUnconfirmedByMember(member, PageUtil.getPageRequest(pageParams));
    }

    public int readUnconfirmedCount(Member member)  throws OverflowException {
        return this.notificationDAO.findAllUnconfirmedCountByMember(member);
    }

    public Page<Notification> markAllAsRead(Member member, PageParams pageParams) throws OverflowException {
        List<Notification> list = this.notificationDAO.findAllByMember(member);
        for (Notification n : list) {
            n.setConfirmDate(new Date());
        }
        this.notificationDAO.save(list);
        return this.readAllByMember(member, pageParams);
    }

    public Page<Notification> markAllAsUnread(Member member, PageParams pageParams) throws OverflowException {
        List<Notification> list = this.notificationDAO.findAllByMember(member);
        for (Notification n : list) {
            n.setConfirmDate(null);
        }
        this.notificationDAO.save(list);
        return this.readAllByMember(member, pageParams);
    }

    public Notification markAsRead(Notification notification) throws OverflowException {
        notification.setConfirmDate(new Date());
        return this.notificationDAO.save(notification);
    }

}
